@@ -1,6 +1,6 @@
 __author__ = "Shyue Ping Ong, William Davidson Richards, Stephen Dacek, " \
              "Xiaohui Qu"
-__date__ = "Jul 14 2014"
-__version__ = "0.7.5"
+__date__ = "Sep 12 2014"
+__version__ = "0.7.6"
 
-from custodian import Custodian
+from .custodian import Custodian
