@@ -14,4 +14,8 @@ __version__ = "0.1"
 __maintainer__ = "Shyue Ping Ong"
 __email__ = "ongsp@ucsd.edu"
 __status__ = "Production"
-__date__ = "Feb 1 2013"
\ No newline at end of file
+__date__ = "Feb 1 2013"
+
+
+from .interpreter import Modder
+from .actions import FileActions, DictActions
\ No newline at end of file
