@@ -7,6 +7,8 @@ ErrorHandlers and Jobs.
 """
 
 from __future__ import division
+import six
+from six.moves import filter
 
 __author__ = "Shyue Ping Ong, William Davidson Richards"
 __copyright__ = "Copyright 2012, The Materials Project"
@@ -28,6 +30,9 @@ import shutil
 from abc import ABCMeta, abstractmethod
 from monty.tempfile import ScratchDir
 from monty.shutil import gzip_dir
+from monty.json import MSONable
+from monty.dev import deprecated
+
 
 pjoin = os.path.join
 
@@ -194,12 +199,14 @@ class Custodian(object):
             else:
                 restart, run_log = -1, []
 
+            terminal = False
+
             for i, job in enumerate(self.jobs):
                 if i <= restart:
                     #Skip all jobs until the restart point.
                     continue
-                run_log.append({"job": job.to_dict, "corrections": []})
-                for attempt in xrange(self.max_errors):
+                run_log.append({"job": job.as_dict(), "corrections": []})
+                for attempt in range(self.max_errors):
                     logger.info(
                         "Starting job no. {} ({}) attempt no. {}. Errors "
                         "thus far = {}.".format(
@@ -225,7 +232,7 @@ class Custodian(object):
                                 if p.poll() is not None:
                                     break
                                 if n % self.monitor_freq == 0:
-                                    corrections = _do_check(
+                                    corrections, terminal = _do_check(
                                         self.monitors,
                                         terminate_func=p.terminate,
                                         skip_over_errors=self.skip_over_errors)
@@ -246,7 +253,7 @@ class Custodian(object):
                     else:
                         remaining_handlers = self.handlers
 
-                    corrections = _do_check(
+                    corrections, terminal = _do_check(
                         remaining_handlers,
                         skip_over_errors=self.skip_over_errors)
                     if len(corrections) > 0:
@@ -265,8 +272,10 @@ class Custodian(object):
                     if not has_error:
                         job.postprocess()
                         break
-                    elif run_log[-1]["corrections"][-1]["actions"] is None:
-                        # Check if there has been an unrecoverable error.
+                    elif not filter(
+                            None,
+                            [x["actions"] for x in run_log[-1]["corrections"]]):
+                        # Check that no corrections were applied
                         logger.info("Unrecoverable error.")
                         unrecoverable = True
                         break
@@ -287,14 +296,14 @@ class Custodian(object):
             run_time = end - start
 
             logger.info("Run completed. Total time taken = {}."
-                         .format(run_time))
+                        .format(run_time))
 
             if self.gzipped_output:
                 gzip_dir(".")
 
-            if total_errors >= self.max_errors or unrecoverable:
+            if terminal and (total_errors >= self.max_errors or unrecoverable):
                 raise RuntimeError("{} errors reached. Unrecoverable? {}. "
-                                   "Exited...".format(self.max_errors,
+                                   "Exited...".format(total_errors,
                                                       unrecoverable))
             elif not unrecoverable:
                 #Cleanup checkpoint files (if any) if run is successful.
@@ -305,9 +314,11 @@ class Custodian(object):
 
 def _do_check(handlers, terminate_func=None, skip_over_errors=False):
     corrections = []
+    terminal = False
     for h in handlers:
         try:
             if h.check():
+                terminal = h.is_terminating
                 if terminate_func is not None and h.is_terminating:
                     terminate_func()
                 d = h.correct()
@@ -320,28 +331,34 @@ def _do_check(handlers, terminate_func=None, skip_over_errors=False):
                 corrections.append(
                     {"errors": ["Bad handler " + str(h)],
                      "actions": []})
-    return corrections
+    return corrections, terminal
 
 
-class JSONSerializable(object):
+class JSONSerializable(MSONable):
     """
     Base class to be inherited to provide useful standard json serialization
     and deserialization protocols based on init args.
     """
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         d = {"@module": self.__class__.__module__,
              "@class": self.__class__.__name__}
         if hasattr(self, "__init__"):
             for c in inspect.getargspec(self.__init__).args:
                 if c != "self":
                     a = self.__getattribute__(c)
-                    if hasattr(a, "to_dict"):
-                        a = a.to_dict
+                    if hasattr(a, "as_dict"):
+                        a = a.as_dict()
                     d[c] = a
         return d
 
+    @property
+    @deprecated(
+        message="All to_dict properties have been deprecated. They will be "
+                "removed from v0.8. Use the as_dict() method instead.")
+    def to_dict(self):
+        return self.as_dict()
+
     @classmethod
     def from_dict(cls, d):
         """
@@ -353,11 +370,10 @@ class JSONSerializable(object):
         return cls(**kwargs)
 
 
-class ErrorHandler(JSONSerializable):
+class ErrorHandler(six.with_metaclass(ABCMeta, JSONSerializable)):
     """
     Abstract base class defining the interface for an ErrorHandler.
     """
-    __metaclass__ = ABCMeta
 
     is_monitor = False
     """
@@ -408,11 +424,10 @@ class ErrorHandler(JSONSerializable):
         pass
 
 
-class Job(JSONSerializable):
+class Job(six.with_metaclass(ABCMeta, JSONSerializable)):
     """
     Abstract base class defining the interface for a Job.
     """
-    __metaclass__ = ABCMeta
 
     @abstractmethod
     def setup(self):
