@@ -5,6 +5,7 @@ Created on Jun 1, 2012
 """
 
 from __future__ import division
+from __future__ import print_function
 
 __author__ = "Shyue Ping Ong"
 __copyright__ = "Copyright 2012, The Materials Project"
@@ -75,13 +76,21 @@ class ExampleHandler2(ErrorHandler):
         return True
 
     def correct(self):
-        return {"errors": "errored", "actions": None}
+        self.has_error = True
+        return {"errors": "Unrecoverable error", "actions": None}
 
     @staticmethod
     def from_dict(d):
-        return ExampleHandler()
+        return ExampleHandler2()
 
 
+class ExampleHandler2b(ExampleHandler2):
+    """
+    This handler always result in an error. But is non-terminating. So no
+    runtime error.
+    """
+    is_terminating = False
+
 class CustodianTest(unittest.TestCase):
 
     def setUp(self):
@@ -92,19 +101,27 @@ class CustodianTest(unittest.TestCase):
         njobs = 100
         params = {"initial": 0, "total": 0}
         c = Custodian([ExampleHandler(params)],
-                      [ExampleJob(i, params) for i in xrange(njobs)],
+                      [ExampleJob(i, params) for i in range(njobs)],
                       max_errors=njobs, log_file=None)
         output = c.run()
         self.assertEqual(len(output), njobs)
-        print ExampleHandler(params).to_dict
+        print(ExampleHandler(params).as_dict())
 
     def test_unrecoverable(self):
         njobs = 100
         params = {"initial": 0, "total": 0}
-        c = Custodian([ExampleHandler2(params)],
-                      [ExampleJob(i, params) for i in xrange(njobs)],
+        h = ExampleHandler2(params)
+        c = Custodian([h],
+                      [ExampleJob(i, params) for i in range(njobs)],
                       max_errors=njobs, log_file=None)
         self.assertRaises(RuntimeError, c.run)
+        self.assertTrue(h.has_error)
+        h = ExampleHandler2b(params)
+        c = Custodian([h],
+                      [ExampleJob(i, params) for i in range(njobs)],
+                      max_errors=njobs, log_file=None)
+        c.run()
+        self.assertTrue(h.has_error)
 
     def tearDown(self):
         for f in glob.glob("custodian.*.tar.gz"):
