@@ -20,10 +20,11 @@ import shutil
 import math
 import logging
 
-from pymatgen.io.vaspio.vasp_input import VaspInput
+from pymatgen.io.vaspio.vasp_input import VaspInput, Incar
+from pymatgen.io.vaspio.vasp_output import Outcar
 from pymatgen.io.smartio import read_structure
 from pymatgen.io.vaspio_set import MITVaspInputSet
-from pymatgen.serializers.json_coders import PMGJSONDecoder
+from monty.json import MontyDecoder
 from monty.os.path import which
 
 from custodian.ansible.interpreter import Modder
@@ -48,7 +49,7 @@ class VaspJob(Job):
                  final=True, gzipped=False, backup=True,
                  default_vasp_input_set=MITVaspInputSet(), auto_npar=True,
                  auto_gamma=True, settings_override=None,
-                 gamma_vasp_cmd=None):
+                 gamma_vasp_cmd=None, copy_magmom=False):
         """
         This constructor is necessarily complex due to the need for
         flexibility. For standard kinds of runs, it's often better to use one
@@ -100,6 +101,10 @@ class VaspJob(Job):
                 auto_gamma is True. Should follow the list style of
                 subprocess. Defaults to None, which means ".gamma" is added
                 to the last argument of the standard vasp_cmd.
+            copy_magmom (bool): Whether to copy the final magmom from the
+                OUTCAR to the next INCAR. Useful for multi-relaxation runs 
+                where the CHGCAR and WAVECAR are sometimes deleted (due to 
+                changes in fft grid, etc.). Only applies to non-final runs.
         """
         self.vasp_cmd = vasp_cmd
         self.output_file = output_file
@@ -112,6 +117,7 @@ class VaspJob(Job):
         self.auto_npar = auto_npar
         self.auto_gamma = auto_gamma
         self.gamma_vasp_cmd = gamma_vasp_cmd
+        self.copy_magmom = copy_magmom
 
     def setup(self):
         """
@@ -139,8 +145,7 @@ class VaspJob(Job):
 
         if self.auto_npar:
             try:
-                vi = VaspInput.from_directory(".")
-                incar = vi["INCAR"]
+                incar = Incar.from_file("INCAR")
                 #Only optimized NPAR for non-HF and non-RPA calculations.
                 if not (incar.get("LHFCALC") or incar.get("LRPA") or
                         incar.get("LEPSILON")):
@@ -199,6 +204,7 @@ class VaspJob(Job):
     def postprocess(self):
         """
         Postprocessing includes renaming and gzipping where necessary.
+        Also copies the magmom to the incar if necessary
         """
         for f in VASP_OUTPUT_FILES + [self.output_file]:
             if os.path.exists(f):
@@ -207,6 +213,16 @@ class VaspJob(Job):
                 elif self.suffix != "":
                     shutil.copy(f, "{}{}".format(f, self.suffix))
 
+        if self.copy_magmom and not self.final:
+            try:
+                outcar = Outcar("OUTCAR")
+                magmom = [m['tot'] for m in outcar.magnetization]
+                incar = Incar.from_file("INCAR")
+                incar['MAGMOM'] = magmom
+                incar.write_file("INCAR")
+            except:
+                logging.error('MAGMOM copy from OUTCAR to INCAR failed')
+
         if self.gzipped:
             gzip_dir(".")
 
@@ -234,12 +250,11 @@ class VaspJob(Job):
                         {"filename": "CONTCAR",
                          "action": {"_file_copy": {"dest": "POSCAR"}}}])]
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         d = dict(vasp_cmd=self.vasp_cmd,
                  output_file=self.output_file, suffix=self.suffix,
                  final=self.final, gzipped=self.gzipped, backup=self.backup,
-                 default_vasp_input_set=self.default_vis.to_dict,
+                 default_vasp_input_set=self.default_vis.as_dict(),
                  auto_npar=self.auto_npar, auto_gamma=self.auto_gamma,
                  settings_override=self.settings_override,
                  gamma_vasp_cmd=self.gamma_vasp_cmd
@@ -250,7 +265,7 @@ class VaspJob(Job):
 
     @classmethod
     def from_dict(cls, d):
-        vis = PMGJSONDecoder().process_decoded(d["default_vasp_input_set"])
+        vis = MontyDecoder().process_decoded(d["default_vasp_input_set"])
         return VaspJob(
             vasp_cmd=d["vasp_cmd"], output_file=d["output_file"],
             suffix=d["suffix"], final=d["final"], gzipped=d["gzipped"],
